"""
Definition for all enneoxOS senosrs 
"""

from .const import Identifier, SMATagList
from .sensor import Sensor

ennexosSensorProfiles = {
    "Sunny Tripower X ": [
        Sensor("Coolsys.Inverter.TmpVal.1", Identifier.temp_a, factor=1, unit="°C"),
        Sensor("Coolsys.Inverter.TmpVal.2", Identifier.temp_b, factor=1, unit="°C"),
        Sensor("Coolsys.Inverter.TmpVal.3", Identifier.temp_c, factor=1, unit="°C"),
        Sensor("DcMs.Amp.1", Identifier.pv_current_a, factor=1, unit="A"),
        Sensor("DcMs.Amp.2", Identifier.pv_current_b, factor=1, unit="A"),
        Sensor("DcMs.Amp.3", Identifier.pv_current_c, factor=1, unit="A"),
        Sensor("DcMs.TotDcEnCntWh.1", None),  # Energy released by string [A]
        Sensor("DcMs.TotDcEnCntWh.2", None, factor=1, unit=None),
        Sensor("DcMs.TotDcEnCntWh.3", None, factor=1, unit=None),
        Sensor("DcMs.Vol.1", Identifier.pv_voltage_a, factor=1, unit="V"),
        Sensor("DcMs.Vol.2", Identifier.pv_voltage_b, factor=1, unit="V"),
        Sensor("DcMs.Vol.3", Identifier.pv_voltage_c, factor=1, unit="V"),
        Sensor("DcMs.Watt.1", Identifier.pv_power_a, factor=1, unit="W"),
        Sensor("DcMs.Watt.2", Identifier.pv_power_b, factor=1, unit="W"),
        Sensor("DcMs.Watt.3", Identifier.pv_power_c, factor=1, unit="W"),
        Sensor("GridGuard.Cntry", None, factor=1, unit=None),  # Country standard set
        Sensor("GridMs.A.phsA", Identifier.current_l1, factor=1, unit="A"),
        Sensor("GridMs.A.phsB", Identifier.current_l2, factor=1, unit="A"),
        Sensor("GridMs.A.phsC", Identifier.current_l3, factor=1, unit="A"),
        Sensor("GridMs.GriTyp", None, factor=1, unit=None),  # Measurement.GridMs.GriTyp
        Sensor("GridMs.Hz", Identifier.frequency, factor=1, unit="Hz"),
        Sensor("GridMs.PhV.phsA", Identifier.voltage_l1, factor=1, unit="V"),
        Sensor(
            "GridMs.PhV.phsA2B", None, factor=1, unit=None
        ),  # Grid voltage phase L1 against L2
        Sensor("GridMs.PhV.phsB", Identifier.voltage_l2, factor=1, unit="V"),
        Sensor(
            "GridMs.PhV.phsB2C", None, factor=1, unit=None
        ),  # Grid voltage phase L2 against L3
        Sensor("GridMs.PhV.phsC", Identifier.voltage_l3, factor=1, unit="V"),
        Sensor(
            "GridMs.PhV.phsC2A", None, factor=1, unit=None
        ),  # Grid voltage phase L3 against L1
        Sensor("GridMs.TotA", Identifier.current_total, factor=1, unit="A"),
        Sensor(
            "GridMs.TotPFEEI", None, factor=1, unit=None
        ),  # EEI displacement power factor
        Sensor(
            "GridMs.TotPFExt", None, factor=1, unit=None
        ),  # Excitation type of cos φ
        Sensor(
            "GridMs.TotPFPrc", None, factor=1, unit=None
        ),  # Displacement power factor
        Sensor("GridMs.TotVA", Identifier.grid_apparent_power, factor=1, unit="VA"),
        Sensor("GridMs.TotVAr", Identifier.grid_reactive_power, factor=1, unit="var"),
        Sensor("GridMs.TotW", Identifier.grid_power, factor=1, unit="W"),
        Sensor("GridMs.TotW.Pv", Identifier.pv_power, factor=1, unit="W"),
        Sensor(
            "GridMs.VA.phsA", Identifier.grid_apparent_power_l1, factor=1, unit="VA"
        ),
        Sensor(
            "GridMs.VA.phsB", Identifier.grid_apparent_power_l2, factor=1, unit="VA"
        ),
        Sensor(
            "GridMs.VA.phsC", Identifier.grid_apparent_power_l3, factor=1, unit="VA"
        ),
        Sensor(
            "GridMs.VAr.phsA", Identifier.grid_reactive_power_l1, factor=1, unit="var"
        ),
        Sensor(
            "GridMs.VAr.phsB", Identifier.grid_reactive_power_l2, factor=1, unit="var"
        ),
        Sensor(
            "GridMs.VAr.phsC", Identifier.grid_reactive_power_l3, factor=1, unit="var"
        ),
        Sensor("GridMs.W.phsA", Identifier.power_l1, factor=1, unit="W"),
        Sensor("GridMs.W.phsB", Identifier.power_l2, factor=1, unit="W"),
        Sensor("GridMs.W.phsC", Identifier.power_l3, factor=1, unit="W"),
        Sensor("InOut.GI1", None, factor=1, unit=None),  # Digital group input
        Sensor("Inverter.VArModCfg.PFCtlVolCfg.Stt", None, factor=1, unit=None),
        Sensor(
            "Isolation.FltA",
            Identifier.insulation_residual_current,
            factor=1000,
            unit="mA",
        ),
        Sensor(
            "Isolation.LeakRis", None, factor=1, unit="kOhm"
        ),  # TODO "pv_isolation_resistance"
        Sensor("Metering.TotFeedTms", None),
        Sensor("Metering.TotOpTms", None),
        Sensor("Metering.TotWhOut", Identifier.total_yield, factor=1000, unit="kWh"),
        Sensor(
            "Metering.TotWhOut.Pv", Identifier.pv_gen_meter, factor=1000, unit="kWh"
        ),
        Sensor("Operation.BckStt", None, factor=1, unit=None),
        Sensor("Operation.DrtStt", None, factor=1, unit=None),
        Sensor("Operation.Evt.Dsc", None, factor=1, unit=None),
        Sensor("Operation.Evt.EvtNo", None, factor=1, unit=None),
        Sensor("Operation.Evt.Msg", None, factor=1, unit=None),
        Sensor("Operation.EvtCntIstl", None, factor=1, unit=None),
        Sensor("Operation.EvtCntUsr", None, factor=1, unit=None),
        Sensor("Operation.GriSwCnt", None, factor=1, unit=None),
        Sensor(
            "Operation.GriSwStt",
            Identifier.grid_relay_status,
            unit=None,
            mapper=SMATagList,
        ),
        Sensor(
            "Operation.Health",
            Identifier.status,
            factor=1,
            unit=None,
            mapper=SMATagList,
        ),
        Sensor("Operation.HealthStt.Alm", None, factor=1, unit=None),
        Sensor("Operation.HealthStt.Ok", None, factor=1, unit=None),
        Sensor("Operation.HealthStt.Wrn", None, factor=1, unit=None),
        Sensor(
            "Operation.OpStt",
            Identifier.operating_status_genereal,
            unit=None,
            mapper=SMATagList,
        ),
        Sensor("Operation.PvGriConn", None, factor=1, unit=None),
        Sensor("Operation.RstrLokStt", None, factor=1, unit=None),
        Sensor("Operation.RunStt", None, factor=1, unit=None),
        Sensor("Operation.StandbyStt", None, factor=1, unit=None),
        Sensor("Operation.VArCtl.VArModAct", None, factor=1, unit=None),
        Sensor("Operation.VArCtl.VArModStt", None, factor=1, unit=None),
        Sensor("Operation.WMaxLimSrc", None, factor=1, unit=None),
        Sensor("Operation.WMinLimSrc", None, factor=1, unit=None),
        Sensor("PvGen.PvW", None, factor=1, unit=None),  #  PV generation power
        Sensor(
            "PvGen.PvWh", None, factor=1, unit=None
        ),  # Meter count and PV gen. meter
        Sensor(
            "Spdwr.ComSocA.Stt", None, factor=1, unit=None
        ),  # Speedwire connection status of SMACOM A
        Sensor(
            "SunSpecSig.SunSpecTx.1", None, factor=1, unit=None
        ),  # SunSpec life sign [1]
        Sensor("Upd.Stt", None, factor=1, unit=None),
        Sensor(
            "WebConn.Stt", None, factor=1, unit=None
        ),  #  Status of the Webconnect functionality
        Sensor("Wl.AcqStt", None, factor=1, unit=None),  # Status of WiFi scan
        Sensor("Wl.AntMod", None, factor=1, unit=None),  #  WiFi antenna type
        Sensor("Wl.ConnStt", None, factor=1, unit=None),  # WiFi connection status
        Sensor(
            "Wl.SigPwr", None, factor=1, unit=None
        ),  # Signal strength of the selected network
        Sensor(
            "Wl.SoftAcsConnStt", None, factor=1, unit=None
        ),  # Soft Access Point status
        Sensor("Setpoint.PlantControl.InOut.GO1", None, factor=1, unit=None),
        Sensor("Inverter.WMax", Identifier.inverter_power_limit, factor=1, unit="W"),
    ],
    "^(SMA EV Charger |EVC22-3AC-10)": [
        Sensor("ChaSess.WhIn", None, factor=1, unit=None),  # charging_session_energy
        Sensor(
            "Chrg.ModSw", None, factor=1, unit=None, mapper=SMATagList
        ),  # position_of_rotary_switch 4950 or 4718
        Sensor(
            "GridMs.A.phsA", Identifier.current_l1, factor=1, unit="A"
        ),  # Netzstrom Phase L1
        Sensor("GridMs.A.phsB", Identifier.current_l2, factor=1, unit="A"),
        Sensor("GridMs.A.phsC", Identifier.current_l3, factor=1, unit="A"),
        Sensor("GridMs.Hz", Identifier.frequency, factor=1, unit="Hz"),
        Sensor(
            "GridMs.PhV.phsA", Identifier.voltage_l1, factor=1, unit="V"
        ),  # Netzspannung Phase L1
        Sensor("GridMs.PhV.phsB", Identifier.voltage_l2, factor=1, unit="V"),
        Sensor("GridMs.PhV.phsC", Identifier.voltage_l3, factor=1, unit="V"),
        Sensor("GridMs.TotPF", None, factor=1, unit=None),
        Sensor("GridMs.TotVA", Identifier.grid_apparent_power, factor=1, unit="VA"),
        Sensor("GridMs.TotVAr", Identifier.grid_reactive_power, factor=1, unit="var"),
        Sensor("InOut.GI1", None, factor=1, unit=None),
        Sensor(
            "Metering.GridMs.TotWIn",
            Identifier.metering_power_absorbed,
            factor=1,
            unit="W",
        ),  #
        Sensor(
            "Metering.GridMs.TotWIn.ChaSta", None, factor=1, unit=None
        ),  # same as Metering.GridMs.TotWIn
        Sensor(
            "Metering.GridMs.TotWhIn",
            Identifier.metering_total_absorbed,
            factor=1000,
            unit="kWh",
        ),  # charging_station_meter_reading
        Sensor(
            "Metering.GridMs.TotWhIn.ChaSta", None, factor=1, unit=None
        ),  # same as Metering.GridMs.TotWhIn
        Sensor(
            "Operation.EVeh.ChaStt",
            Identifier.operating_status,
            factor=1,
            unit=None,
            mapper=SMATagList,
        ),  # charging_session_status
        Sensor(
            "Operation.EVeh.Health",
            Identifier.status,
            factor=1,
            unit=None,
            mapper=SMATagList,
        ),
        Sensor("Operation.Evt.Msg", None, factor=1, unit=None),
        Sensor("Operation.Health", None, factor=1, unit=None),  # same as EVeh.Health?
        Sensor("Operation.WMaxLimNom", None, factor=1, unit=None),
        Sensor("Operation.WMaxLimSrc", None, factor=1, unit=None),
        Sensor("Wl.AcqStt", None, factor=1, unit=None),
        Sensor("Wl.ConnStt", None, factor=1, unit=None),
        Sensor("Wl.SigPwr", None, factor=1, unit=None),
        Sensor("Wl.SoftAcsConnStt", None, factor=1, unit=None),
    ],
    "Sunny Boy Smart Energy ": [
        Sensor("Coolsys.Inverter.TmpVal.1", Identifier.temp_a, factor=1, unit="°C"),
        Sensor("DcMs.Amp.1", Identifier.pv_current_a, factor=1, unit="A"),
        Sensor("DcMs.Amp.2", Identifier.pv_current_b, factor=1, unit="A"),
        Sensor("DcMs.Amp.3", Identifier.pv_current_c, factor=1, unit="A"),
        Sensor("DcMs.TotDcEnCntWh.1", None),  # Energy released by string [A]
        Sensor("DcMs.TotDcEnCntWh.2", None, factor=1, unit=None),
        Sensor("DcMs.TotDcEnCntWh.3", None, factor=1, unit=None),
        Sensor("DcMs.Vol.1", Identifier.pv_voltage_a, factor=1, unit="V"),
        Sensor("DcMs.Vol.2", Identifier.pv_voltage_b, factor=1, unit="V"),
        Sensor("DcMs.Vol.3", Identifier.pv_voltage_c, factor=1, unit="V"),
        Sensor("DcMs.Watt.1", Identifier.pv_power_a, factor=1, unit="W"),
        Sensor("DcMs.Watt.2", Identifier.pv_power_b, factor=1, unit="W"),
        Sensor("DcMs.Watt.3", Identifier.pv_power_c, factor=1, unit="W"),
        Sensor("GridGuard.Cntry", None, factor=1, unit=None),  # Country standard set
        Sensor("GridMs.A.phsA", Identifier.current_l1, factor=1, unit="A"),
        Sensor(
            "GridMs.A.phsA", Identifier.current_l1, factor=1, unit="A"
        ),  # Netzstrom Phase L1
        Sensor("GridMs.GriTyp", None, factor=1, unit=None),  # Measurement.GridMs.GriTyp
        Sensor("GridMs.Hz", Identifier.frequency, factor=1, unit="Hz"),
        Sensor(
            "GridMs.PhV.phsA2B", None, factor=1, unit=None
        ),  # Grid voltage phase L1 against L2
        Sensor("GridMs.PhV.phsA", Identifier.voltage_l1, factor=1, unit="V"),
        Sensor("GridMs.TotA", Identifier.current_total, factor=1, unit="A"),
        Sensor(
            "GridMs.TotPFEEI", None, factor=1, unit=None
        ),  # EEI displacement power factor
        Sensor(
            "GridMs.TotPFExt", None, factor=1, unit=None
        ),  # Excitation type of cos φ
        Sensor(
            "GridMs.TotPFPrc", None, factor=1, unit=None
        ),  # Displacement power factor
        Sensor("GridMs.TotVA", Identifier.grid_apparent_power, factor=1, unit="VA"),
        Sensor("GridMs.TotVAr", Identifier.grid_reactive_power, factor=1, unit="var"),
        Sensor("GridMs.TotW", Identifier.grid_power, factor=1, unit="W"),
        Sensor("GridMs.TotW.Pv", Identifier.pv_power, factor=1, unit="W"),
        Sensor(
            "GridMs.VA.phsA", Identifier.grid_apparent_power_l1, factor=1, unit="VA"
        ),
        Sensor(
            "GridMs.VAr.phsA", Identifier.grid_reactive_power_l1, factor=1, unit="var"
        ),
        Sensor("GridMs.W.phsA", Identifier.power_l1, factor=1, unit="W"),
        Sensor("InOut.GI1", None, factor=1, unit=None),
        Sensor("Inverter.VArModCfg.PFCtlVolCfg.Stt", None, factor=1, unit=None),
        Sensor(
            "Isolation.FltA",
            Identifier.insulation_residual_current,
            factor=1000,
            unit="mA",
        ),
        Sensor(
            "Isolation.LeakRis", None, factor=1, unit="kOhm"
        ),  # TODO "pv_isolation_resistance"
        Sensor(
            "Metering.GridMs.TotWhIn.ChaSta", None, factor=1, unit=None
        ),  # same as Metering.GridMs.TotWhIn
        Sensor(
            "Metering.GridMs.TotWhIn",
            Identifier.metering_total_absorbed,
            factor=1000,
            unit="kWh",
        ),  # charging_station_meter_reading
        Sensor(
            "Metering.GridMs.TotWIn.ChaSta", None, factor=1, unit=None
        ),  # same as Metering.GridMs.TotWIn
        Sensor(
            "Metering.GridMs.TotWIn",
            Identifier.metering_power_absorbed,
            factor=1,
            unit="W",
        ),  #
        Sensor("Metering.TotFeedTms", None),
        Sensor("Metering.TotOpTms", None),
        Sensor("Metering.TotWhOut", Identifier.total_yield, factor=1000, unit="kWh"),
        Sensor(
            "Metering.TotWhOut.Pv", Identifier.pv_gen_meter, factor=1000, unit="kWh"
        ),
        Sensor("Operation.BckStt", None, factor=1, unit=None),
        Sensor("Operation.DrtStt", None, factor=1, unit=None),
        Sensor("Operation.EvtCntIstl", None, factor=1, unit=None),
        Sensor("Operation.EvtCntUsr", None, factor=1, unit=None),
        Sensor("Operation.Evt.Dsc", None, factor=1, unit=None),
        Sensor("Operation.Evt.Msg", None, factor=1, unit=None),
        Sensor("Operation.GriSwCnt", None, factor=1, unit=None),
        Sensor(
            "Operation.GriSwStt",
            Identifier.grid_relay_status,
            unit=None,
            mapper=SMATagList,
        ),
        Sensor(
            "Operation.Health",
            Identifier.status,
            factor=1,
            unit=None,
            mapper=SMATagList,
        ),
        Sensor("Operation.HealthStt.Alm", None, factor=1, unit=None),
        Sensor("Operation.HealthStt.Ok", None, factor=1, unit=None),
        Sensor("Operation.HealthStt.Wrn", None, factor=1, unit=None),
        Sensor(
            "Operation.OpStt",
            Identifier.operating_status_genereal,
            unit=None,
            mapper=SMATagList,
        ),
        Sensor("Operation.PvGriConn", None, factor=1, unit=None),
        Sensor("Operation.RstrLokStt", None, factor=1, unit=None),
        Sensor("Operation.RunStt", None, factor=1, unit=None),
        Sensor("Operation.StandbyStt", None, factor=1, unit=None),
        Sensor("Operation.VArCtl.VArModAct", None, factor=1, unit=None),
        Sensor("Operation.WMaxLimSrc", None, factor=1, unit=None),
        Sensor(
            "PvGen.PvWh", None, factor=1, unit=None
        ),  # Meter count and PV gen. meter
        Sensor("PvGen.PvW", None, factor=1, unit=None),  #  PV generation power
        Sensor("Setpoint.PlantControl.InOut.GO1", None, factor=1, unit=None),
        Sensor(
            "Spdwr.ComSocA.Stt", None, factor=1, unit=None
        ),  # Speedwire connection status of SMACOM A
        Sensor(
            "SunSpecSig.SunSpecTx.1", None, factor=1, unit=None
        ),  # SunSpec life sign [1]
        Sensor("Upd.Stt", None, factor=1, unit=None),
        Sensor(
            "WebConn.Stt", None, factor=1, unit=None
        ),  #  Status of the Webconnect functionality
        Sensor("Wl.AcqStt", None, factor=1, unit=None),
        Sensor("Wl.AntMod", None, factor=1, unit=None),  #  WiFi antenna type
        Sensor("Wl.ConnStt", None, factor=1, unit=None),
        Sensor("Wl.SigPwr", None, factor=1, unit=None),
        Sensor("Wl.SoftAcsConnStt", None, factor=1, unit=None),
        Sensor("Bat.Amp.1", None, factor=1, unit=None),
        Sensor("Bat.ChaSttArr.1", None, factor=1, unit=None),
        Sensor("Bat.Diag.VolMax.1", None, factor=1, unit=None),
        Sensor("Bat.OpTms.1", None, factor=1, unit=None),  #### Tot. battery runtime
        Sensor("Bat.Vol.1", None, factor=1, unit=None),
        Sensor("BatChrg.BatChrg", None, factor=1, unit=None),  #### Battery charge
        Sensor("BatChrg.BatChrgArr.1", None, factor=1, unit=None),  ####
        Sensor(
            "BatChrg.CurBatChaArr.1", None, factor=1, unit=None
        ),  ### Present battery charge
        Sensor("BatDsch.BatDsch", None, factor=1, unit=None),  ###
        Sensor("BatDsch.BatDschArr.1", None, factor=1, unit=None),  ####
        Sensor("BatDsch.CurBatDschArr.1", None, factor=1, unit=None),
        Sensor("DcMs.TotDcEnCntWh.1", None, factor=1, unit=None),
        Sensor("DcMs.TotDcEnCntWh.2", None, factor=1, unit=None),
        Sensor("DcMs.TotDcEnCntWh.3", None, factor=1, unit=None),
        Sensor("EpsMs.Amp", None, factor=1, unit=None),  # Emergency power measurements
        Sensor("EpsMs.Vol", None, factor=1, unit=None),  # Emergency power measurements
        Sensor("EpsMs.Watt", None, factor=1, unit=None),  # Emergency power measurements
        Sensor("GridGuard.Cntry", None, factor=1, unit=None),
        Sensor("GridMs.GriTyp", None, factor=1, unit=None),
        Sensor("GridMs.TotPFEEI", None, factor=1, unit=None),
        Sensor("GridMs.TotPFExt", None, factor=1, unit=None),
        Sensor("GridMs.TotPFPrc", None, factor=1, unit=None),
        Sensor("InOut.GI1", None, factor=1, unit=None),
        Sensor("InOut.GO1", None, factor=1, unit=None),
        Sensor("Inverter.VArModCfg.PFCtlVolCfg.Stt", None, factor=1, unit=None),
        Sensor("Isolation.LeakRis", None, factor=1, unit=None),
        Sensor("Metering.GridMs.TotWhIn.Bat", None, factor=1, unit=None),
        Sensor("Metering.GridMs.TotWhOut.Bat", None, factor=1, unit=None),
        Sensor("Metering.TotFeedTms", None, factor=1, unit=None),
        Sensor("Metering.TotOpTms", None, factor=1, unit=None),
        Sensor("Metering.TotWhIn", None, factor=1, unit=None),
        Sensor("Operation.Bat.Health", None, factor=1, unit=None),
        Sensor("Operation.BckStt", None, factor=1, unit=None),
        Sensor("Operation.DrtStt", None, factor=1, unit=None),
        Sensor("Operation.Evt.Dsc", None, factor=1, unit=None),
        Sensor("Operation.Evt.Msg", None, factor=1, unit=None),
        Sensor("Operation.EvtCntIstl", None, factor=1, unit=None),
        Sensor("Operation.EvtCntUsr", None, factor=1, unit=None),
        Sensor("Operation.GriSwCnt", None, factor=1, unit=None),
        Sensor("Operation.HealthStt.Alm", None, factor=1, unit=None),
        Sensor("Operation.HealthStt.Ok", None, factor=1, unit=None),
        Sensor("Operation.HealthStt.Wrn", None, factor=1, unit=None),
        Sensor("Operation.PvGriConn", None, factor=1, unit=None),
        Sensor("Operation.RstrLokStt", None, factor=1, unit=None),
        Sensor("Operation.RunStt", None, factor=1, unit=None),
        Sensor("Operation.StandbyStt", None, factor=1, unit=None),
        Sensor("Operation.VArCtl.VArModAct", None, factor=1, unit=None),
        Sensor("Operation.VArCtl.VArModStt", None, factor=1, unit=None),
        Sensor("Operation.WMaxLimSrc", None, factor=1, unit=None),
        Sensor("Operation.WMinLimSrc", None, factor=1, unit=None),
        Sensor("Portal.EnnexOS.ConnStt", None, factor=1, unit=None),
        Sensor("PvGen.PvW", None, factor=1, unit=None),
        Sensor("PvGen.PvWh", None, factor=1, unit=None),
        Sensor("Spdwr.ComSocA.Stt", None, factor=1, unit=None),
        Sensor("SunSpecSig.SunSpecTx.1", None, factor=1, unit=None),
        Sensor("Upd.Stt", None, factor=1, unit=None),
        Sensor("WebConn.Stt", None, factor=1, unit=None),
        Sensor("Wl.AcqStt", None, factor=1, unit=None),
        Sensor("Wl.AntMod", None, factor=1, unit=None),
        Sensor("Wl.ConnStt", None, factor=1, unit=None),
        Sensor("Wl.SigPwr", None, factor=1, unit=None),
        Sensor("Wl.SoftAcsConnStt", None, factor=1, unit=None),
        Sensor("Setpoint.PlantControl.CmpBMS.OpMod.1", None, factor=1, unit=None),
        Sensor("Metering.GridMs.TotWhIn.Bat", None, factor=1, unit=None),
        Sensor("Metering.GridMs.TotWhOut.Bat", None, factor=1, unit=None),
    ],
}



"""
abstract base class on which all device implementations are based
"""

from abc import ABC, abstractmethod
from typing import Any, Dict, List

from .sensor import Sensor, Sensors


class Device(ABC):
    """abstract base class on which all device implementations are based."""

    @abstractmethod
    async def get_sensors(self) -> Sensors:
        """Returns a list of all supported sensors"""

    @abstractmethod
    async def new_session(self) -> bool:
        """Starts a new session"""

    @abstractmethod
    async def device_info(self) -> dict:
        """Return a Dict with basic device information"""

    @abstractmethod
    async def read(self, sensors: Sensors) -> bool:
        """Updates all sensors"""

    @abstractmethod
    async def close_session(self) -> None:
        """Closes the session"""

    async def detect(self, ip: str) -> List:
        """Tries an automatic detection of this device and returns the results"""
        return [
            {
                "testedEndpoints": "",
                "status": "failed",
                "access": "",
                "device": "",
                "exception": "",
                "remark": "",
                "ip": ip,
            }
        ]

    @abstractmethod
    async def get_debug(self) -> Dict[str, Any]:
        """Return a dict with all debug information."""

    def set_options(self, options: Dict[str, Any]):
        """Set options"""
        pass

    async def handleModulActions(self):
        pass

    def set_parameter(self, sensor: Sensor | str, value: int):
        pass



""" Speedwire Discovery  """

import asyncio
import logging
import socket
import struct

from .definitions_speedwire import speedwireHeader

_LOGGER = logging.getLogger(__name__)


class Discovery:
    def __init__(self, loop):
        self.loop = loop
        self.transport = None
        self.addr = "239.12.255.254"
        self.port = 9522
        self.discovered = []

    def getDiscoverySocket(self):
        addrinfo = socket.getaddrinfo(self.addr, None)[0]
        sock = socket.socket(addrinfo[0], socket.SOCK_DGRAM)
        sock.setsockopt(
            socket.IPPROTO_IP, socket.IP_MULTICAST_TTL, struct.pack("@i", 1)
        )
        return sock

    async def run(self):
        sock = self.getDiscoverySocket()
        on_connection_lost = self.loop.create_future()
        connect = await self.loop.create_datagram_endpoint(
            lambda: self,
            sock=sock,
        )
        for i in range(0, 3):
            await asyncio.sleep(0.5)
            self.sendDiscoveryRequest()
        await asyncio.sleep(0.5)
        return self.discovered

    def connection_made(self, transport):
        self.transport = transport
        self.sendDiscoveryRequest()

    def sendDiscoveryRequest(self):
        _LOGGER.warn("Sending Discovery Request")
        self.transport.sendto(
            bytes.fromhex("534d4100000402a0ffffffff0000002000000000"),
            (self.addr, self.port),
        )

    def datagram_received(self, data, addr):
        msg = speedwireHeader.from_packed(data[0:18])
        if not msg.isDiscoveryResponse():
            _LOGGER.warn("Ignoring %", msg)
            return
        if addr not in self.discovered:
            self.discovered.append(addr)

    def error_received(self, exc):
        _LOGGER.error("Error received:", exc)

    def connection_lost(self, exc):
        print("Socket closed, stop the event loop")



"""

Interface for SMA ennoxOS based devices. (e.g. Tripower X and maybe EV Charger)

"""

import asyncio
import copy
import json
import logging
import re
from datetime import UTC, datetime
from typing import Any, Dict, Optional

from aiohttp import ClientSession, ClientTimeout, client_exceptions, hdrs

from .const import SMATagList
from .const_webconnect import DEFAULT_TIMEOUT
from .definitions_ennexos import ennexosSensorProfiles
from .device import Device
from .exceptions import (
    SmaAuthenticationException,
    SmaConnectionException,
    SmaReadException,
)
from .sensor import Sensor_Range, Sensors

_LOGGER = logging.getLogger(__name__)


class SMAennexos(Device):
    """Class to connect to the ennexos based SMA inverters. (e.g. Tripower X Devices)"""

    # pylint: disable=too-many-instance-attributes
    _aio_session: ClientSession
    _new_session_data: Optional[dict]
    _url: str
    _token: str
    _authorization_header: dict[str, str]
    _last_parameters: Any = {}
    _last_measurements: Any = {}
    _last_device: Any = {}
    _last_notfound: list = []
    _device_info: Dict = None
    _jsessionid: str = None
    _options: Dict[str, Any] = {}

    def __init__(
        self,
        session: ClientSession,
        url: str,
        password: Optional[str],
        group: str,
    ):
        """Init SMA connection.

        Args:
            session (ClientSession): aiohttp client session
            url (str): Url or IP address of device
            password (str, optional): Password to use during login.
            group (str, optional): Username to use during login.

        """
        self._url = url.rstrip("/")
        if not url.startswith("http"):
            self._url = "https://" + self._url
        self._new_session_data = {"user": group, "pass": password}
        self._aio_session = session

    async def _jsonrequest(
        self, url: str, parameters: Dict[str, Any], method: str = hdrs.METH_POST
    ) -> dict:
        """Request json data for requests.

        Args:
            url (str): URL to do request to
            parameters (Dict[str, Any]): parameters
            method (str): Post or Get-Request

        Raises:
            SmaConnectionException: Connection to device failed
            SmaAuthenticationException: Authentication failed

        Returns:
            dict: json returned by device
        """

        # _LOGGER.debug("Sending Request to %s: %s", url, parameters)

        try:
            async with self._aio_session.request(
                method, url, timeout=ClientTimeout(total=DEFAULT_TIMEOUT), **parameters
            ) as res:
                if res.status == 401:
                    raise SmaAuthenticationException("Token failed!")
                res = await res.json()
                # _LOGGER.debug("Received reply %s", res)
                return res
        except SmaAuthenticationException as e:
            raise e
        except (client_exceptions.ContentTypeError, json.decoder.JSONDecodeError):
            _LOGGER.warning("Request to %s did not return a valid json.", url)
        except client_exceptions.ServerDisconnectedError as exc:
            raise SmaConnectionException(
                f"Server at {self._url} disconnected."
            ) from exc
        except (
            client_exceptions.ClientError,
            asyncio.exceptions.TimeoutError,
        ) as exc:
            raise SmaConnectionException(
                f"Could not connect to SMA at {self._url}: {exc}"
            ) from exc
        return {}

    async def new_session(self) -> bool:
        """Establish a new session.

        Returns:
            bool: authentication successful
        """
        loginurl = self._url + "/api/v1/token"
        postdata = {
            "data": {
                "grant_type": "password",
                "username": self._new_session_data["user"],
                "password": self._new_session_data["pass"],
            }
        }
        ret = await self._jsonrequest(loginurl, postdata)
        if "access_token" not in ret:
            raise SmaAuthenticationException("Login failed!")
        self._token = ret["access_token"]
        #        self._refreshtoken = ret["refresh_token"]
        self._authorization_header = {
            "Authorization": "Bearer " + self._token,
            "Content-Type": "application/json",
        }
        _LOGGER.debug("Login successfull")
        return True

    async def _get_parameter(self) -> Dict:
        """Get all parameters from the device

        Returns:
            Dict: Return a dict with all parameters

        """
        url = self._url + "/api/v1/parameters/search"
        postdata = {
            "data": '{"queryItems":[{"componentId":"IGULD:SELF"}]}',
            "headers": self._authorization_header,
        }
        ret = await self._jsonrequest(url, postdata)
        data = {}
        if len(ret) != 1:
            _LOGGER.warning(
                "Uncommon length of array in parameters request: %d", len(ret)
            )

        for d in ret[0]["values"]:
            dname = d["channelId"].replace("Parameter.", "").replace("[]", "")
            if "value" in d:
                v = d["value"]
                sensorRange = None
                if "min" in d and "max" in d:
                    sensorRange = Sensor_Range(
                        "min/max", [d["min"], d["max"]], d["editable"]
                    )
                if "possibleValues" in d:
                    sensorRange = Sensor_Range(
                        "selection", d["possibleValues"], d["editable"]
                    )

                data[dname] = {
                    "name": dname,
                    "value": v,
                    "origname": d["channelId"],
                    "range": sensorRange,
                }

            elif "values" in d:
                # Split Value-Arrays
                for idx in range(0, len(d["values"])):
                    v = d["values"][idx]
                    idxname = dname + "." + str(idx + 1)
                    data[idxname] = {
                        "name": idxname,
                        "value": v,
                        "origname": d["channelId"],
                    }
            else:
                # Value current not available // night?
                pass
        return data

    async def _get_livedata(self) -> Dict:
        """Get the sensors reading from the device

        Returns:
            Dict: Return a dict with all measurements

        """
        liveurl = self._url + "/api/v1/measurements/live"
        postdata = {
            "data": '[{"componentId":"IGULD:SELF"}]',
            "headers": self._authorization_header,
        }
        ret = await self._jsonrequest(liveurl, postdata)
        return await self._prepare_livedata(ret)

    async def _prepare_livedata(self, ret):
        self._last_measurements = ret
        data = {}
        for d in ret:
            dname = d["channelId"].replace("Measurement.", "").replace("[]", "")
            if "value" in d["values"][0]:
                v = d["values"][0]["value"]
                if self._isfloat(v):
                    v = round(v, 2)
                data[dname] = {"name": dname, "value": v, "origname": d["channelId"]}
            elif "values" in d["values"][0]:
                # Split Value-Arrays
                for idx in range(0, len(d["values"][0]["values"])):
                    v = d["values"][0]["values"][idx]
                    if self._isfloat(v):
                        v = round(v, 2)
                    idxname = dname + "." + str(idx + 1)
                    data[idxname] = {
                        "name": idxname,
                        "value": v,
                        "origname": d["channelId"],
                    }
            else:
                # Value current not available // night?
                pass
        return data

    async def get_sensors(self) -> Sensors:
        """Get the sensors that are present on the device.

        Returns:
            Sensors: Sensors object containing Sensor objects
        """
        if not self._device_info:
            raise SmaReadException("device_info() not called!")

        ret = await self._get_livedata() | await self._get_parameter()
        _LOGGER.debug("Found Sensors: %s", ret)
        profile = await self._get_sensor_profile()
        return profile

    async def _get_sensor_profile(self):
        device_sensors = Sensors()
        expectedSensors = []

        # Search for matiching profile
        for profil in ennexosSensorProfiles.items():
            if re.search(profil[0], self._device_info["name"]):
                expectedSensors = profil[1]
        if len(expectedSensors) == 0:
            _LOGGER.warning(
                f'Unknown Device: {self._device_info["name"]} {self._device_info["type"]}'
            )

        # Add Sensors from profile
        for s in expectedSensors:
            if s.name:
                device_sensors.add(copy.copy(s))
        return device_sensors

    async def close_session(self) -> None:
        pass

    def _isfloat(self, num: Any):
        """Test if num is a float

            Tests for type float or a string with a dot is is float

        Args:
            num: number to check

        Returns:
            bool: true, if num is from type float or a string with a dot
        """
        if isinstance(num, float):
            return True
        if isinstance(num, int):
            return False
        if not isinstance(num, str):
            raise TypeError("Value is not a string, float or int!")
        if "." not in num:
            return False
        try:
            float(num)
            return True
        except ValueError:
            return False

    async def read(self, sensors: Sensors) -> bool:
        """Read a set of keys.

        Args:
            sensors (Sensors): Sensors object containing Sensor objects to read

        Returns:
            bool: reading was successful
        """
        notfound = []
        data = None
        try:
            data = await self._get_livedata() | await self._get_parameter()
        except SmaAuthenticationException:
            # Relogin
            _LOGGER.debug("Re-login .. Starting new Session")
            await self.new_session()
            data = await self._get_livedata() | await self._get_parameter()
        for sen in sensors:
            if sen.enabled:
                if sen.key in data:
                    value = data[sen.key]["value"]
                    if sen.mapper:
                        sen.mapped_value = sen.mapper.get(value, str(value))
                    if sen.factor and sen.factor != 1:
                        value = round(value / sen.factor, 4)
                    sen.value = value
                    if "range" in data[sen.key]:
                        sen.range = data[sen.key]["range"]
                    continue
                notfound.append(f"{sen.name} [{sen.key}]")

        self._last_notfound = notfound
        if notfound:
            _LOGGER.info(
                "No values for sensors: %s",
                ",".join(notfound),
            )

        return True

    async def device_info(self) -> dict:
        """Read device info and return the results.

        Returns:
            dict: dict containing serial, name, type, manufacturer and sw_version
        """
        url = self._url + "/api/v1/plants/Plant:1/devices/IGULD:SELF"
        requestdata = {"headers": self._authorization_header}
        dev = await self._jsonrequest(url, requestdata, hdrs.METH_GET)
        self._last_device = dev
        self._last_parameters = await self._get_parameter()
        _LOGGER.debug("Found Device: %s", dev)
        self._device_info = {
            "serial": dev["serial"],
            "name": dev["product"],
            "type": dev["name"],
            "manufacturer": dev["vendor"],
            "sw_version": dev["firmwareVersion"],
        }
        return self._device_info

    async def get_debug(self) -> Dict:
        """Collect all Debug Information"""
        return {
            "device": self._last_device,
            "measurements": self._last_measurements,
            "parameters": self._last_parameters,
            "device_info": self._device_info,
            "notfound": self._last_notfound,
        }

    async def detect(self, ip):
        rets = []
        for prefix in ["https", "http"]:
            url = f"{prefix}://{ip}/api/v1/system/info"
            ret = (await super().detect(ip))[0]
            rets.append(ret)
            ret["testedEndpoints"] = url
            ret["remark"] = prefix
            try:
                dev = await self._jsonrequest(url, {}, hdrs.METH_GET)
                if "productFriendlyNameTagId" in dev:
                    fallback = "Unkown: " + str(dev["productFriendlyNameTagId"])
                    ret["device"] = SMATagList.get(
                        dev["productFriendlyNameTagId"], fallback
                    )
                    ret["status"] = "found"
                    break
                ret["status"] = "failed"
                ret["exception"] = dev
            except Exception as e:
                ret["status"] = "failed"
                ret["exception"] = e
        return rets

    def set_options(self, options: Dict[str, Any]):
        self._options = options

    async def getTimestamp(self):
        return (
            f"{datetime.now(tz=UTC).isoformat(timespec='milliseconds').split('+')[0]}Z"
        )

    # async def handleModulActions(self):
    #     if self._options.get("CMD", "") != "SET":
    #         return
    #     channelName = self._options.get("NAME", "")
    #     sensors = await self.get_sensors()
    #     for sensor in sensors:
    #         sensor.enabled = True
    #     await self.read(sensors)
    #     for sensor in sensors:
    #         if (sensor.name == channelName):
    #             found = sensor
    #     if not found:
    #         _LOGGER.error("Sensor {} not found!".format(channelName))
    #         return

    #     sensorData = await self._get_parameter()
    #     sensorData = sensorData[found.key]
    #     timestamp = await self.getTimestamp()
    #     value = self._options.get("VALUE","")
    #     channelName = sensorData["origname"]
    #     url = self._url + "/api/v1/parameters/IGULD:SELF"
    #     requestData = f'{{"values":[{{"channelId":"{channelName}","timestamp":"{timestamp}","value":"{value}"}}]}}'
    #     putdata = {
    #         "data": requestData,
    #         "headers": self._authorization_header,
    #     }
    #     print(putdata)
    #     dev = await self._jsonrequest(url, putdata, hdrs.METH_PUT)
    #     print(dev)



"""Interface for SMA Energy Meters and Sunny Home Manager 2 (SHM2)

see https://www.unifox.at/software/sma-em-daemon/
see https://cdn.sma.de/fileadmin/content/www.developer.sma.de/docs/EMETER-Protokoll-TI-en-10.pdf?v=1699276024

"""

import base64
import copy
import datetime
import logging
import socket
import struct
from typing import Any, Dict

from .const import Identifier
from .device import Device
from .exceptions import SmaConnectionException, SmaReadException
from .sensor import Sensor, Sensors

obis2sensor = [
    Sensor(
        "1:4:0", Identifier.metering_power_absorbed, factor=10, unit="W"
    ),  # p consume
    Sensor("1:8:0", Identifier.metering_total_absorbed, factor=3600000, unit="kWh"),
    Sensor(
        "2:4:0", Identifier.metering_power_supplied, factor=10, unit="W"
    ),  # p supply
    Sensor(
        "2:8:0",
        Identifier.metering_total_yield,
        factor=3600000,
        unit="kWh",
    ),
    Sensor("3:4:0", None),  # q consume
    Sensor("3:8:0", None),
    Sensor("4:4:0", None),  # q supply
    Sensor("4:8:0", None),
    Sensor("9:4:0", None),  # s consume
    Sensor("9:8:0", None),
    Sensor("10:4:0", None),  # s supply
    Sensor("10:8:0", None),
    Sensor("13:4:0", None),  # cospi
    Sensor("14:4:0", Identifier.metering_frequency, factor=1000, unit="Hz"),  # freq
    # Phase 1
    Sensor("21:4:0", Identifier.metering_active_power_draw_l1, factor=10, unit="W"),
    Sensor("21:8:0", None),
    Sensor("22:4:0", Identifier.metering_active_power_feed_l1, factor=10, unit="W"),
    Sensor("22:8:0", None),
    Sensor("23:4:0", None),
    Sensor("23:8:0", None),
    Sensor("24:4:0", None),
    Sensor("24:8:0", None),
    Sensor("29:4:0", None),
    Sensor("29:8:0", None),
    Sensor("30:4:0", None),
    Sensor("30:8:0", None),
    Sensor("31:4:0", Identifier.metering_current_l1, factor=1000, unit="A"),
    Sensor("32:4:0", Identifier.metering_voltage_l1, factor=1000, unit="V"),
    Sensor("33:4:0", None),  # cosphi1
    # Phase 2
    Sensor("41:4:0", Identifier.metering_active_power_draw_l2, factor=10, unit="W"),
    Sensor("41:8:0", None),
    Sensor("42:4:0", Identifier.metering_active_power_feed_l2, factor=10, unit="W"),
    Sensor("42:8:0", None),
    Sensor("43:4:0", None),
    Sensor("43:8:0", None),
    Sensor("44:4:0", None),
    Sensor("44:8:0", None),
    Sensor("49:4:0", None),
    Sensor("49:8:0", None),
    Sensor("50:4:0", None),
    Sensor("50:8:0", None),
    Sensor("51:4:0", Identifier.metering_current_l2, factor=1000, unit="A"),
    Sensor("52:4:0", Identifier.metering_voltage_l2, factor=1000, unit="V"),
    Sensor("53:4:0", None),
    # Phase 3
    Sensor("61:4:0", Identifier.metering_active_power_draw_l3, factor=10, unit="W"),
    Sensor("61:8:0", None),
    Sensor("62:4:0", Identifier.metering_active_power_feed_l3, factor=10, unit="W"),
    Sensor("62:8:0", None),
    Sensor("63:4:0", None),
    Sensor("63:8:0", None),
    Sensor("64:4:0", None),
    Sensor("64:8:0", None),
    Sensor("69:4:0", None),
    Sensor("69:8:0", None),
    Sensor("70:4:0", None),
    Sensor("70:8:0", None),
    Sensor("71:4:0", Identifier.metering_current_l3, factor=1000, unit="A"),
    Sensor("72:4:0", Identifier.metering_voltage_l3, factor=1000, unit="V"),
    Sensor("73:4:0", None),
]


_LOGGER = logging.getLogger(__name__)


class SMAspeedwireEM(Device):
    """Class to connect to the ennexos based SMA inverters. (e.g. Tripower X Devices)"""

    _sock: socket
    _susyid: Dict[int, Any] = {
        270: "Energy Meter",
        349: "Energy Meter 2",
        372: "Sunny Home Manager 2",
    }
    _last_packet: bytes = None

    def __init__(self):
        """Init SMA connection.

        Args:
            session (ClientSession): aiohttp client session
            url (str): Url or IP address of device
            password (str, optional): Password to use during login.
            group (str, optional): Username to use during login.

        """

    async def new_session(self) -> bool:
        """Establish a new session.

        Returns:
            bool: authentication successful
        """
        mcast_grp = "239.12.255.254"
        mcast_port = 9522
        ipbind = "0.0.0.0"

        self._sock = socket.socket(
            socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP
        )
        self._sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self._sock.settimeout(5)
        self._sock.bind(("", mcast_port))
        try:
            mreq = struct.pack(
                "4s4s", socket.inet_aton(mcast_grp), socket.inet_aton(ipbind)
            )
            self._sock.setsockopt(socket.IPPROTO_IP, socket.IP_ADD_MEMBERSHIP, mreq)
        except BaseException as exc:
            raise SmaConnectionException("Could not start multicast") from exc

        return True

    async def get_sensors(self) -> Sensors:
        """Get the sensors that are present on the device.

        Returns:
            Sensors: Sensors object containing Sensor objects
        """
        device_sensors = Sensors()

        for s in obis2sensor:
            if s.name is not None:
                device_sensors.add(copy.copy(s))

        return device_sensors

    async def close_session(self) -> None:
        """Close the session login."""
        self._sock.close()

    def _recv(self):
        return self._sock.recv(608)

    def _get_data(self):
        """

        Hack:
            If the function is called less frequently than the device supplies data,
            the UDP packets will be stored in the buffer.
            If the read instruction (sock.recv) takes only milliseconds, I assume that
            the data came from the buffer and discard it. Otherwise outdated values would
            be used.
            One of my most ugly hacks.

        """
        data = None
        tries = 50
        try:
            while tries > 0:
                tries -= 1
                a = datetime.datetime.now()
                self._last_packet = self._recv()
                data = self._decode(self._last_packet)
                b = datetime.datetime.now()
                if data and (b - a).total_seconds() < 0.1:
                    continue
                if data:
                    break
        except TimeoutError as e:
            raise SmaConnectionException("No speedwire packet received!") from e
        if not data:
            raise SmaReadException("No usable data received!")
        return data

    async def read(self, sensors: Sensors) -> bool:
        """Read a set of keys.

        Args:
            sensors (Sensors): Sensors object containing Sensor objects to read

        Returns:
            bool: reading was successful
        """
        notfound = []
        data = self._get_data()

        for sensor in sensors:
            if sensor.key in data:
                value = data[sensor.key]
                if sensor.factor:
                    value /= sensor.factor
                sensor.value = value
            else:
                notfound.append(sensor.key)

        if notfound:
            _LOGGER.info(
                "No values for sensors: %s",
                ",".join(notfound),
            )

        return True

    async def device_info(self) -> dict:
        """Read device info and return the results.

        Returns:
            dict: dict containing serial, name, type, manufacturer and sw_version
        """
        data = self._get_data()

        device_info = {
            "serial": data["serial"],
            "name": data["device"],
            "type": data["susyid"],
            "manufacturer": "SMA",
            "sw_version": data["sw_version"],
        }
        return device_info

    def _decode(self, p: bytes):
        """Decode a Speedwire-Packet

        Args:
            p: Network-Packet

        Returns:
            dict: Dict with all the decoded information
        """
        if p[0:4] != b"SMA\0":
            return None
        protocol_id = int.from_bytes(p[16:18], byteorder="big")

        if protocol_id not in [0x6069, 0x6081]:
            _LOGGER.debug("Unknown protocoll %d", protocol_id)
            return None

        data = {}
        data["protocolID"] = protocol_id
        data["susyid"] = int.from_bytes(p[18:20], byteorder="big")
        data["device"] = self._susyid.get(data["susyid"], "unknown")
        data["serial"] = int.from_bytes(p[20:24], byteorder="big")

        length = int.from_bytes(p[12:14], byteorder="big") + 16
        pos = 28
        while pos < length:
            value = None
            mchannel = int.from_bytes(p[pos : pos + 1], byteorder="big")
            mvalueindex = int.from_bytes(p[pos + 1 : pos + 2], byteorder="big")
            mtyp = int.from_bytes(p[pos + 2 : pos + 3], byteorder="big")
            mtariff = int.from_bytes(p[pos + 3 : pos + 4], byteorder="big")
            obis = f"{mvalueindex}:{mtyp}:{mtariff}"
            if mtyp in [4, 8]:
                # 4 actucal / current => 8 Bytes
                # 8 counter / sum => 12 Bytes
                value = int.from_bytes(p[pos + 4 : pos + 4 + mtyp], byteorder="big")
                pos += 4 + mtyp
            elif mchannel == 144 and mtyp == 0:
                value = f"{p[pos + 4]}.{p[pos + 5]}.{p[pos + 6]}.{chr(p[pos + 7])}"
                obis = "sw_version"
                pos += 4 + 4
            else:
                _LOGGER.debug(
                    "Unknown packet in speedwire: "
                    + str(mchannel)
                    + " "
                    + str(mvalueindex)
                    + " "
                    + str(mtyp)
                    + " "
                    + str(mtariff)
                )
                pos += 4 + 4
            data[obis] = value
        return data

    async def get_debug(self) -> Dict:
        encoded = base64.b64encode(self._last_packet)
        return {"packet": encoded.decode("ascii")}



from pysma.definitions_speedwire import commands

class Test_speedwire_class:
    """Test the Speedwire class."""

    async def test_unique_responses(self):
        response = set()
        duplicates = set()
        for r in commands.values():
          #  print(r)
            resp = f'{r["response"]:X}'
            if  resp in response:
                duplicates.add(resp)
            else:
                response.add(resp)
        print(duplicates)
        assert len(duplicates) == 0


    # async def test_unique_command(self):
    #     cmds = set()
    #     duplicates = set()
    #     for r in commands.values():
    #       #  print(r)
    #         resp = f'{r["command"]:X}'
    #         if  resp in cmds:
    #             duplicates.add(resp)
    #         else:
    #             cmds.add(resp)
    #     print(duplicates)
    #     assert len(duplicates) == 0
